"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createServer = exports.Ecosystem = void 0;
var channels_1 = require("./channels");
var log_1 = require("./log");
var server_1 = require("./server");
var subscribers_1 = require("./subscribers");
var Ecosystem = (function () {
    function Ecosystem(options) {
        if (options === void 0) { options = {}; }
        this.options = options;
        this.defaultOptions = {
            app: {
                id: '',
                secret: ''
            },
            dev: true,
            subscribers: {
                http: true,
                redis: true,
                nats: true,
            },
            socketio: {},
            server: {},
            redis: {
                options: {
                    host: '0.0.0.0',
                    port: 6379
                }
            }
        };
        this.clients = [];
        this.nsps = [];
        this.options = Object.assign(this.defaultOptions, this.options);
        this.startup();
        this.server = new server_1.Server(this.options);
    }
    Ecosystem.prototype.channel = function (channel) {
        this.options.channel = channel;
        return this;
    };
    Ecosystem.prototype.of = function (name) {
        if (typeof name === 'string' && !name.startsWith('/')) {
            name = "/" + name;
        }
        this.nsps.push({ name: name, clients: [] });
        return this;
    };
    Ecosystem.prototype.listen = function () {
        var _this = this;
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        return new Promise(function (resolve, reject) { return __awaiter(_this, void 0, void 0, function () {
            var _this = this;
            return __generator(this, function (_a) {
                this.server.init().then(function (timer) {
                    setTimeout(function () { return __awaiter(_this, void 0, void 0, function () {
                        var _a;
                        return __generator(this, function (_b) {
                            switch (_b.label) {
                                case 0: return [4, (_a = this.server.app).listen.apply(_a, args)];
                                case 1:
                                    _b.sent();
                                    timer && clearInterval(timer);
                                    this.init();
                                    resolve({
                                        io: Object.assign(this.server.io, { nsps: this.nsps }),
                                        http: this.server.app
                                    });
                                    log_1.Log.info('\n');
                                    log_1.Log.removeUp(2);
                                    log_1.Log.info('Server ready!\n');
                                    return [2];
                            }
                        });
                    }); }, 3500);
                }).catch(function (error) { return reject(error); });
                return [2];
            });
        }); });
    };
    Ecosystem.prototype.init = function () {
        var _this = this;
        this.buildChannel();
        return new Promise(function (resolve, reject) {
            _this.subscribers = [];
            if (_this.options.subscribers.http)
                _this.subscribers.push(new subscribers_1.HttpSubscriber(_this.server.app, _this.options));
            if (_this.options.subscribers.redis)
                _this.subscribers.push(new subscribers_1.RedisSubscriber(_this.server.redis, _this.options));
            if (_this.options.subscribers.nats)
                _this.subscribers.push(new subscribers_1.NatsSubscriber(_this.server.nats, _this.options));
            _this.onConnect();
            _this.run().then(resolve, reject);
        });
    };
    Ecosystem.prototype.startup = function () {
        log_1.Log.title('\nE C O S Y S T E M  S O C K E T  S E R V E R\n\n');
        if (this.options.dev) {
            log_1.Log.warning('Starting server in DEV mode\n');
        }
        else {
            log_1.Log.info('Starting server\n');
        }
    };
    Ecosystem.prototype.run = function () {
        var _this = this;
        return new Promise(function (resolve, reject) {
            Promise.all(_this.subscribers.map(function (subscriber, index) {
                return subscriber.subscribe(function (channel, message, name) {
                    _this.broadcast(channel, message, name);
                });
            })).then(resolve, reject);
        });
    };
    Ecosystem.prototype.broadcast = function (channel, message, name) {
        var socket = this.findSocketById(message.socket, name);
        if (socket) {
            this.others(socket, channel, message);
        }
        else {
            message.socket = undefined;
            this.all(channel, message, name);
        }
    };
    Ecosystem.prototype.findSocketById = function (id, name) {
        var nsp = this.findNspByName(name);
        if (!!nsp) {
            return nsp.clients.find(function (val) { return val.id === id; });
        }
        return this.clients.find(function (val) { return val.id === id; });
    };
    Ecosystem.prototype.findNspByName = function (name) {
        var nsp = this.nsps.find(function (val) { return new RegExp(val.name).test(name); });
        return nsp !== null && nsp !== void 0 ? nsp : null;
    };
    Ecosystem.prototype.others = function (socket, channel, message) {
        socket.broadcast.to(channel).emit(message.event, message);
    };
    Ecosystem.prototype.all = function (channel, message, name) {
        var nsp = this.findNspByName(name);
        var socket = !!nsp ? nsp.socket : this.server.io;
        socket.to(channel).emit(message.event, message);
    };
    Ecosystem.prototype.onConnect = function () {
        var _this = this;
        this.nsps.forEach(function (value, index) {
            _this.nsps[index].socket = _this.server.io.of(value.name, function (socket) {
                _this.nsps[index].clients.push(socket);
                _this.onSubscribe(socket);
                _this.onUnsubscribe(socket);
                _this.onDisconnecting(socket, false, index);
            }).use(function (req, next) { return _this.server.authorizeRequests(req.handshake, next); });
        });
        this.server.io.on('connection', function (socket) { return __awaiter(_this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                this.clients.push(socket);
                this.onSubscribe(socket);
                this.onUnsubscribe(socket);
                this.onDisconnecting(socket);
                return [2];
            });
        }); });
    };
    Ecosystem.prototype.onSubscribe = function (socket) {
        var _this = this;
        socket.on('subscribe', function (data) {
            _this.options.channel.join(socket, data);
        });
    };
    Ecosystem.prototype.onUnsubscribe = function (socket) {
        var _this = this;
        socket.on('unsubscribe', function (data) {
            _this.options.channel.leave(socket, data.channel, 'unsubscribed');
        });
    };
    Ecosystem.prototype.onDisconnecting = function (socket, nsp, index) {
        var _this = this;
        socket.on('disconnect', function (reason) {
            if (!nsp) {
                _this.clients = _this.clients.filter(function (v) { return v.id !== socket.id; });
            }
            else {
                _this.nsps[index].clients =
                    _this.nsps[index].clients.filter(function (v) { return v.id === socket.id; });
            }
            _this.options.channel.leave(socket, socket.id, socket.nsp.name + " - " + reason);
        });
    };
    Ecosystem.prototype.buildChannel = function () {
        if (this.options.channel === undefined) {
            this.options.channel = new channels_1.Channel();
        }
        else {
            this.options.channel = typeof this.options.channel === 'function'
                ? this.options.channel(this.server.io) : this.options.channel;
            if (typeof this.options.channel !== 'object') {
                this.options.channel = undefined;
            }
            else {
                if (!this.options.channel.join &&
                    !this.options.channel.leave) {
                    this.options.channel = undefined;
                }
            }
        }
    };
    return Ecosystem;
}());
exports.Ecosystem = Ecosystem;
function createServer(opts) {
    return new Ecosystem(opts);
}
exports.createServer = createServer;
//# sourceMappingURL=index.js.map