/// <reference types="node" />
import { FastifyServerOptions } from 'fastify';
import io from 'socket.io';
import { SecureContextOptions } from 'tls';
import { ChannelInterface } from './channels';
import { CorsOptions } from 'cors';
import { RedisClient } from 'redis';
import { Client, NatsConnectionOptions } from 'ts-nats';
interface AppServer {
    id: string;
    secret: string;
}
interface ServerOptions extends FastifyServerOptions {
    cors?: CorsOptions;
}
interface NatsOptions extends NatsConnectionOptions {
    prefix?: string;
}
export interface Options {
    app: AppServer;
    dev: boolean;
    channel?: ChannelInterface;
    subscribers: {
        http: boolean;
        redis: boolean;
        nats: boolean;
    };
    socketio: Partial<io.ServerOptions>;
    ssl?: SecureContextOptions;
    server: ServerOptions;
    redis: {
        client?: any;
        options?: any;
        prefix?: string;
    };
    nats?: NatsOptions;
}
export declare class Server {
    private options;
    app: any;
    io: io.Server;
    redis: RedisClient;
    nats: Client;
    sockets: any;
    constructor(options: Options);
    init(): Promise<NodeJS.Timeout>;
    authorizeRequests(req: any, next: any): void;
    private checkRegisteredApp;
    private registerPlugins;
    private middlewares;
}
export {};
//# sourceMappingURL=server.d.ts.map